var teamspeak = require('node-teamspeak');
var events = require('events');
var util = require('util');

function Alfred() {
    var self = this;
    var eventMap = {'message': 'textmessage', 'join': 'cliententerview', 'leave': 'clientleftview'};
    var eventMapInv = {};
    
    self.config = {};
    self.stack = [];
    self.globals = {};
    self.registered = [];
    self.connected = false;
    
    Alfred.prototype.login = function(options, callbackFunction) {
        var self = this;
        self.config = options;
        
        self.teamspeak = new teamspeak(options.host);
        
        self.teamspeak.send('login', {'client_login_name': options.loginName, 'client_login_password': options.loginPass}, function(err, res, raw) {
            if(err) throw new Error(util.inspect(err));

            self.teamspeak.send('use', {'sid': options.sid}, function(err, res, raw) {
                if(err) throw new Error(util.inspect(err));
                
                self.sendCommand = self.teamspeak.send;
                
                for(var i=0; i<self.registered.length; i++) self.teamspeak.send('servernotifyregister', self.registered[i]);
                
                self.teamspeak.send('whoami', null, function(err, res, raw) {
                    if(err) throw new Error(util.inspect(err));
                    self.teamspeak.send('clientupdate', {'client_nickname': options.name});
                    self.clid = res['client_id'];
                });
                
                for(var i=0; i<self.stack.length; i++) {
                    if(typeof self.stack[i].callback.login === 'function') self.stack[i].callback.login.call(self);
                }
                
                for(var eventAlias in eventMap) {
                    var curr = eventAlias;
                    
                    eventMapInv[eventMap[curr]] = curr;
                    
                    self.teamspeak.on(eventMap[eventAlias], function(res) {
                        var event = eventMapInv[Object.keys(res)[0]];
                        
                        self.callStack(event, res);
                    });
                }
                
                self.connected = true;
                if(typeof callbackFunction == 'function') callbackFunction.call(this);
            });
        });
    }
    
    Alfred.prototype.use = function() {
        var self = this;
        var event = typeof arguments[0] === 'string' ? arguments[0] : '<all>';
        var extension = typeof arguments[1] === 'function' ? arguments[1] : arguments[0];
        
        if(extension.init) extension.init.call(self);
        
        self.stack.push({
            'event': event,
            'callback': extension
        });
    }
    
    Alfred.prototype.set = function(key, value) {
        return this.globals[key] = value;
    }
    
    Alfred.prototype.get = function(key) {
        return this.globals[key];
    }
    
    Alfred.prototype.callStack = function(event, data, callbackFunction) {
        var self = this;
        var stackController = new events.EventEmitter();
        var globalData = data;
        
        stackController.on('next', function(i) {
            if(i > self.stack.length - 1) {
                if(typeof callbackFunction == 'function') return callbackFunction();
                return;
            }

            if(self.stack[i].event != event && self.stack[i].event != '<all>') return stackController.emit('next', i + 1);;
            
            self.stack[i].callback.call(self, event, globalData, function() {
                stackController.emit('next', i + 1);
            });
        });
        
        stackController.emit('next', 0); 
    }
    
    Alfred.prototype.registerEvent = function(event, id) {
        if(typeof id === 'number') self.registered.push({'event': event, 'id': id});
        else self.registered.push({'event': event});

        if(self.connected && typeof id === 'number') self.teamspeak.send('servernotifyregister', {'event': event, 'id': id});
        else if(self.connected) self.teamspeak.send('servernotifyregister', {'event': event});
        
        return this;
    }
}

module.exports = function() {
    var retAlfred = new Alfred();

    retAlfred.use(module.exports.User);
    retAlfred.use('message', module.exports.cmd);
    
    return retAlfred;
};

module.exports.Base = Alfred;
module.exports.User = require('./core/user.js');
module.exports.cmd = require('./core/cmd.js');